import pytest

from registrar.config import BackendConfig, HandlerConfig, HandlersConfig, RegistrarConfig, RouteConfig, RouteMode, SourceConfig
from registrar.backend import get_backends
from registrar.registrar import register

from testapp.backends import MyItemBackend, MyPathBackend
from testapp import handlers


TEST_ITEM = """{
    "type": "Feature",
    "stac_version": "1.0",
    "id": "test",
    "geometry": null,
    "properties": {
        "datetime": "2021-11-3T00:00:00Z"
    },
    "links": {

    },
    "assets": {

    }
}"""


# def test_get_backends():
#     config = {
#         "sources": [
#             {
#                 "name": "my-local-source",
#                 "type": "local",
#                 "kwargs": {"root_directory": "/opt/data"},
#             }
#         ],
#         "backends": [
#             {"path": "testapp.backends.MyItemBackend", "kwargs": {"exists": True}}
#         ],
#     }

#     backends = get_backends(config)
#     assert len(backends) == 1
#     assert isinstance(backends[0], MyItemBackend)
#     assert backends[0].exists


def test_register_item(mocker):
    mocker.patch("testapp.backends.MyItemBackend.exists")
    mocker.patch("testapp.backends.MyItemBackend.register")

    config = RegistrarConfig(
        {
            "items": RouteConfig(
                "registrar.route.stac.ItemRoute",
                "register",
                replace=True,
                backends=[
                    BackendConfig(
                        "testapp.backends.MyItemBackend",
                        args=[],
                        kwargs={
                            "exists": True
                        },
                    )
                ],
                # handlers=HandlersConfig(
                #     [HandlerConfig("some.pre.handler")],
                #     [HandlerConfig("some.post.handler")],
                #     [HandlerConfig("some.error.handler")],
                # )
            ),
        },
        [
            SourceConfig(
                type="local",
                name="my-local-source",
                kwargs={"root_directory": "/opt/data"},
            )
        ]
    )
    register(config.routes["items"], config.sources, TEST_ITEM)

    MyItemBackend.exists.assert_called_once()
    MyItemBackend.register.assert_called_once()


def test_handlers(mocker):
    mocker.patch("testapp.handlers.pre_handler.__call__")
    mocker.patch("testapp.handlers.post_handler.__call__")
    mocker.patch("testapp.handlers.error_handler.__call__")

    config = RegistrarConfig(
        {
            "items": RouteConfig(
                "registrar.route.stac.ItemRoute",
                "register",
                replace=True,
                backends=[
                    BackendConfig(
                        "testapp.backends.MyItemBackend",
                        args=[],
                        kwargs={
                            "exists": True
                        },
                    )
                ],
                handlers=HandlersConfig(
                    [HandlerConfig("testapp.handlers.pre_handler")],
                    [HandlerConfig("testapp.handlers.post_handler")],
                    [HandlerConfig("testapp.handlers.error_handler")],
                )
            ),
        },
        [
            SourceConfig(
                type="local",
                name="my-local-source",
                kwargs={"root_directory": "/opt/data"},
            )
        ]
    )
    register(config.routes["items"], config.sources, TEST_ITEM)
    handlers.pre_handler.__call__.assert_called_once()
    handlers.post_handler.__call__.assert_called_once()
